@@ -9,11 +9,11 @@ Test::WWW::Mechanize - Testing-specific WWW::Mechanize subclass
 
 =head1 VERSION
 
-Version 1.31_01
+Version 1.45_01
 
 =cut
 
-our $VERSION = '1.31_01';
+our $VERSION = '1.45_01';
 
 =head1 SYNOPSIS
 
@@ -72,7 +72,7 @@ use Carp::Assert::More;
 
 use base 'WWW::Mechanize';
 
-my $Test = Test::Builder->new();
+my $TB = Test::Builder->new();
 
 
 =head1 CONSTRUCTOR
@@ -84,7 +84,10 @@ passed in get passed to WWW::Mechanize's constructor.
 
 You can pass in C<< autolint => 1 >> to make Test::WWW::Mechanize
 automatically run HTML::Lint after any of the following methods are
-called.
+called. You can also pass in an HTML::Lint object like this:
+
+    my $lint = HTML::Lint->new( only_types => HTML::Lint::Error::STRUCTURE );
+    my $mech = Test::WWW::Mechanize->new( autolint => $lint );
 
 =over
 
@@ -114,6 +117,8 @@ and can simply do
 The C<< $mech->get_ok() >> only counts as one test in the test count.  Both the
 main IO operation and the linting must pass for the entire test to pass.
 
+You can control autolint on the fly with the C<< autolint >> method.
+
 =cut
 
 sub new {
@@ -128,7 +133,7 @@ sub new {
 
     my $self = $class->SUPER::new( %args );
 
-    $self->{autolint} = $autolint;
+    $self->autolint( $autolint );
 
     return $self;
 }
@@ -167,17 +172,17 @@ sub _maybe_lint {
     local $Test::Builder::Level = $Test::Builder::Level + 1;
 
     if ( $ok ) {
-        if ( $self->is_html && $self->{autolint} ) {
+        if ( $self->is_html && $self->autolint ) {
             $ok = $self->_lint_content_ok( $desc );
         }
         else {
-            $Test->ok( $ok, $desc );
+            $TB->ok( $ok, $desc );
         }
     }
     else {
-        $Test->ok( $ok, $desc );
-        $Test->diag( $self->status );
-        $Test->diag( $self->response->message ) if $self->response;
+        $TB->ok( $ok, $desc );
+        $TB->diag( $self->status );
+        $TB->diag( $self->response->message ) if $self->response;
     }
 
     return $ok;
@@ -202,10 +207,10 @@ sub head_ok {
     $self->head( $url, %opts );
     my $ok = $self->success;
 
-    $Test->ok( $ok, $desc );
+    $TB->ok( $ok, $desc );
     if ( !$ok ) {
-        $Test->diag( $self->status );
-        $Test->diag( $self->response->message ) if $self->response;
+        $TB->diag( $self->status );
+        $TB->diag( $self->response->message ) if $self->response;
     }
 
     return $ok;
@@ -254,15 +259,61 @@ sub put_ok {
     $self->put( $url, %opts );
 
     my $ok = $self->success;
-    $Test->ok( $ok, $desc );
+    $TB->ok( $ok, $desc );
     if ( !$ok ) {
-        $Test->diag( $self->status );
-        $Test->diag( $self->response->message ) if $self->response;
+        $TB->diag( $self->status );
+        $TB->diag( $self->response->message ) if $self->response;
     }
 
     return $ok;
 }
 
+=head2 $mech->delete_ok( $url, [ \%LWP_options ,] $desc )
+
+A wrapper around WWW::Mechanize's delete(), with similar options, except
+the second argument needs to be a hash reference, not a hash. Like
+well-behaved C<*_ok()> functions, it returns true if the test passed,
+or false if not.
+
+A default description of "DELETE to $url" is used if none if provided.
+
+=cut
+
+sub delete_ok {
+    my $self = shift;
+
+    my ($url,$desc,%opts) = $self->_unpack_args( 'DELETE', @_ );
+
+    if ($self->can('delete')) {
+        $self->delete( $url, %opts );
+    }
+    else {
+        # When version of LWP::UserAgent is older than 6.04.
+        $self->_delete( $url, %opts );
+    }
+    my $ok = $self->success;
+
+    $ok = $self->_maybe_lint( $ok, $desc );
+
+    return $ok;
+}
+
+sub _delete {
+    require URI;
+    require HTTP::Request::Common;
+    my $self = shift;
+    my $uri  = shift;
+
+    $uri = $uri->url if ref($uri) eq 'WWW::Mechanize::Link';
+    $uri = $self->base
+      ? URI->new_abs( $uri, $self->base )
+      : URI->new($uri);
+
+    my @parameters = ( $uri->as_string, @_ );
+    my @suff = $self->_process_colonic_headers( \@parameters, 1 );
+    return $self->request( HTTP::Request::Common::DELETE(@parameters), @suff );
+}
+
 =head2 $mech->submit_form_ok( \%parms [, $desc] )
 
 Makes a C<submit_form()> call and executes tests on the results.
@@ -296,26 +347,13 @@ sub submit_form_ok {
     my $desc = shift;
 
     if ( ref $parms ne 'HASH' ) {
-       Carp::croak 'FATAL: parameters must be given as a hashref';
+        Carp::croak 'FATAL: parameters must be given as a hashref';
     }
 
     # return from submit_form() is an HTTP::Response or undef
     my $response = $self->submit_form( %{$parms} );
 
-    my $ok;
-    my $error;
-    if ( !$response ) {
-        $error = 'No matching form found';
-    }
-    else {
-        if ( $response->is_success ) {
-            $ok = 1;
-        }
-        else {
-            $error = $response->as_string;
-        }
-    }
-
+    my $ok = $response && $response->is_success;
     $ok = $self->_maybe_lint( $ok, $desc );
 
     return $ok;
@@ -360,20 +398,7 @@ sub follow_link_ok {
     # return from follow_link() is an HTTP::Response or undef
     my $response = $self->follow_link( %{$parms} );
 
-    my $ok;
-    my $error;
-    if ( !$response ) {
-        $error = 'No matching link found';
-    }
-    else {
-        if ( $response->is_success ) {
-            $ok = 1;
-        }
-        else {
-            $error = $response->as_string;
-        }
-    }
-
+    my $ok = $response && $response->is_success;
     $ok = $self->_maybe_lint( $ok, $desc );
 
     return $ok;
@@ -394,7 +419,7 @@ sub click_ok {
 
     my $response = $self->click( $button );
     if ( !$response ) {
-        return $Test->ok( 0, $desc );
+        return $TB->ok( 0, $desc );
     }
 
 
@@ -468,8 +493,8 @@ sub html_lint_ok {
         $ok = $self->_lint_content_ok( $desc );
     }
     else {
-        $ok = $Test->ok( 0, $desc );
-        $Test->diag( q{This page doesn't appear to be HTML, or didn't get the proper text/html content type returned.} );
+        $ok = $TB->ok( 0, $desc );
+        $TB->diag( q{This page doesn't appear to be HTML, or didn't get the proper text/html content type returned.} );
     }
 
     return $ok;
@@ -481,26 +506,34 @@ sub _lint_content_ok {
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
 
-    if ( not ( eval 'require HTML::Lint' ) ) {
-        die "Test::WWW::Mechanize can't do linting without HTML::Lint: $@";
+    my $module = "HTML::Lint 2.20";
+    if ( not ( eval "use $module; 1;" ) ) {
+        die "Test::WWW::Mechanize can't do linting without $module: $@";
+    }
+
+    my $lint = $self->{autolint};
+    if ( ref $lint && $lint->isa('HTML::Lint') ) {
+        $lint->newfile;
+        $lint->clear_errors;
+    }
+    else {
+        $lint = HTML::Lint->new();
     }
 
-    # XXX Combine with the cut'n'paste version in get_ok()
-    my $lint = HTML::Lint->new;
     $lint->parse( $self->content );
 
     my @errors = $lint->errors;
     my $nerrors = @errors;
     my $ok;
     if ( $nerrors ) {
-        $ok = $Test->ok( 0, $desc );
-        $Test->diag( 'HTML::Lint errors for ' . $self->uri );
-        $Test->diag( $_->as_string ) for @errors;
+        $ok = $TB->ok( 0, $desc );
+        $TB->diag( 'HTML::Lint errors for ' . $self->uri );
+        $TB->diag( $_->as_string ) for @errors;
         my $s = $nerrors == 1 ? '' : 's';
-        $Test->diag( "$nerrors error$s on the page" );
+        $TB->diag( "$nerrors error$s on the page" );
     }
     else {
-        $ok = $Test->ok( 1, $desc );
+        $ok = $TB->ok( 1, $desc );
     }
 
     return $ok;
@@ -643,8 +676,9 @@ sub content_contains {
     my $desc = shift;
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
-    if ( ref($str) eq 'REGEX' ) {
-        diag( 'content_contains takes a string, not a regex' );
+
+    if ( ref($str) ) {
+        return $TB->ok( 0, 'Test::WWW::Mechanize->content_contains called incorrectly.  It requires a scalar, not a reference.' );
     }
     $desc = qq{Content contains "$str"} if !defined($desc);
 
@@ -663,8 +697,8 @@ sub content_lacks {
     my $desc = shift;
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
-    if ( ref($str) eq 'REGEX' ) {
-        diag( 'content_lacks takes a string, not a regex' );
+    if ( ref($str) ) {
+        return $TB->ok( 0, 'Test::WWW::Mechanize->content_lacks called incorrectly.  It requires a scalar, not a reference.' );
     }
     $desc = qq{Content lacks "$str"} if !defined($desc);
 
@@ -714,7 +748,7 @@ C<text_contains> instead of L</content_contains>.
  $mech->content_contains('Hi, User'); # Fails.
  $mech->text_contains('Hi, User'); # Passes.
 
-Text is determined by calling C<< $mech->content(format => 'text') >>.
+Text is determined by calling C<< $mech->text() >>.
 See L<WWW::Mechanize/content>.
 
 =cut
@@ -725,11 +759,31 @@ sub text_contains {
     my $desc = shift || qq{Text contains "$str"};
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
-    if ( ref($str) eq 'REGEX' ) {
-        diag( 'text_contains takes a string, not a regex' );
+    if ( ref($str) ) {
+        return $TB->ok( 0, 'Test::WWW::Mechanize->text_contains called incorrectly.  It requires a scalar, not a reference.' );
     }
 
-    return contains_string( $self->content(format => "text"), $str, $desc );
+    return contains_string( $self->text, $str, $desc );
+}
+
+=head2 $mech->text_lacks( $str [, $desc ] )
+
+Tells if the text of the page lacks I<$str>.
+
+=cut
+
+sub text_lacks {
+    my $self = shift;
+    my $str = shift;
+    my $desc = shift;
+
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    if ( ref($str) ) {
+        return $TB->ok( 0, 'Test::WWW::Mechanize->text_lacks called incorrectly.  It requires a scalar, not a reference.' );
+    }
+    $desc = qq{Text lacks "$str"} if !defined($desc);
+
+    return lacks_string( $self->text, $str, $desc );
 }
 
 =head2 $mech->text_like( $regex [, $desc ] )
@@ -776,7 +830,7 @@ sub has_tag {
 
     my $found = $self->_tag_walk( $tag, sub { $text eq $_[0] } );
 
-    return $Test->ok( $found, $desc );
+    return $TB->ok( $found, $desc );
 }
 
 
@@ -795,7 +849,7 @@ sub has_tag_like {
 
     my $found = $self->_tag_walk( $tag, sub { $_[0] =~ $regex } );
 
-    return $Test->ok( $found, $desc );
+    return $TB->ok( $found, $desc );
 }
 
 
@@ -807,7 +861,7 @@ sub _tag_walk {
     my $p = HTML::TokeParser->new( \($self->content) );
 
     while ( my $token = $p->get_tag( $tag ) ) {
-        my $tagtext = $p->get_trimmed_text( "/$tag" );
+        my $tagtext = $p->get_trimmed_text();
         return 1 if $match->( $tagtext );
     }
     return;
@@ -823,7 +877,7 @@ https links.
 sub followable_links {
     my $self = shift;
 
-    return $self->find_all_links( url_abs_regex => qr{^https?://} );
+    return $self->find_all_links( url_abs_regex => qr{^(?:https?|file)://} );
 }
 
 =head2 $mech->page_links_ok( [ $desc ] )
@@ -846,8 +900,8 @@ sub page_links_ok {
     my @failures = $self->_check_links_status( \@urls );
     my $ok = (@failures==0);
 
-    $Test->ok( $ok, $desc );
-    $Test->diag( $_ ) for @failures;
+    $TB->ok( $ok, $desc );
+    $TB->diag( $_ ) for @failures;
 
     return $ok;
 }
@@ -868,10 +922,11 @@ sub page_links_content_like {
 
     $desc = qq{All links are like "$regex"} unless defined $desc;
 
-    my $usable_regex=$Test->maybe_regex( $regex );
-    unless(defined( $usable_regex )) {
-        my $ok = $Test->ok( 0, 'page_links_content_like' );
-        $Test->diag(qq{     "$regex" doesn't look much like a regex to me.});
+    my $usable_regex=$TB->maybe_regex( $regex );
+
+    if ( !defined( $usable_regex ) ) {
+        my $ok = $TB->ok( 0, 'page_links_content_like' );
+        $TB->diag(qq{     "$regex" doesn't look much like a regex to me.});
         return $ok;
     }
 
@@ -881,8 +936,8 @@ sub page_links_content_like {
     my @failures = $self->_check_links_content( \@urls, $regex );
     my $ok = (@failures==0);
 
-    $Test->ok( $ok, $desc );
-    $Test->diag( $_ ) for @failures;
+    $TB->ok( $ok, $desc );
+    $TB->diag( $_ ) for @failures;
 
     return $ok;
 }
@@ -901,12 +956,13 @@ sub page_links_content_unlike {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
-    $desc = "All links are unlike '$regex'" if !defined($desc);
+    $desc = qq{All links are unlike "$regex"} unless defined($desc);
 
-    my $usable_regex=$Test->maybe_regex( $regex );
-    unless(defined( $usable_regex )) {
-        my $ok = $Test->ok( 0, 'page_links_content_unlike' );
-        $Test->diag(qq{     "$regex" doesn't look much like a regex to me.});
+    my $usable_regex=$TB->maybe_regex( $regex );
+
+    if ( !defined( $usable_regex ) ) {
+        my $ok = $TB->ok( 0, 'page_links_content_unlike' );
+        $TB->diag(qq{     "$regex" doesn't look much like a regex to me.});
         return $ok;
     }
 
@@ -916,8 +972,8 @@ sub page_links_content_unlike {
     my @failures = $self->_check_links_content( \@urls, $regex, 'unlike' );
     my $ok = (@failures==0);
 
-    $Test->ok( $ok, $desc );
-    $Test->diag( $_ ) for @failures;
+    $TB->ok( $ok, $desc );
+    $TB->diag( $_ ) for @failures;
 
     return $ok;
 }
@@ -949,8 +1005,8 @@ sub links_ok {
     my @failures = $self->_check_links_status( \@urls );
     my $ok = (@failures == 0);
 
-    $Test->ok( $ok, $desc );
-    $Test->diag( $_ ) for @failures;
+    $TB->ok( $ok, $desc );
+    $TB->diag( $_ ) for @failures;
 
     return $ok;
 }
@@ -979,8 +1035,8 @@ sub link_status_is {
     my @failures = $self->_check_links_status( \@urls, $status );
     my $ok = (@failures == 0);
 
-    $Test->ok( $ok, $desc );
-    $Test->diag( $_ ) for @failures;
+    $TB->ok( $ok, $desc );
+    $TB->diag( $_ ) for @failures;
 
     return $ok;
 }
@@ -1009,8 +1065,8 @@ sub link_status_isnt {
     my @failures = $self->_check_links_status( \@urls, $status, 'isnt' );
     my $ok = (@failures == 0);
 
-    $Test->ok( $ok, $desc );
-    $Test->diag( $_ ) for @failures;
+    $TB->ok( $ok, $desc );
+    $TB->diag( $_ ) for @failures;
 
     return $ok;
 }
@@ -1035,10 +1091,11 @@ sub link_content_like {
     my $regex = shift;
     my $desc = shift;
 
-    my $usable_regex=$Test->maybe_regex( $regex );
-    unless(defined( $usable_regex )) {
-        my $ok = $Test->ok( 0, 'link_content_like' );
-        $Test->diag(qq{     "$regex" doesn't look much like a regex to me.});
+    my $usable_regex=$TB->maybe_regex( $regex );
+
+    if ( !defined( $usable_regex ) ) {
+        my $ok = $TB->ok( 0, 'link_content_like' );
+        $TB->diag(qq{     "$regex" doesn't look much like a regex to me.});
         return $ok;
     }
 
@@ -1047,8 +1104,8 @@ sub link_content_like {
     my @failures = $self->_check_links_content( \@urls, $regex );
     my $ok = (@failures == 0);
 
-    $Test->ok( $ok, $desc );
-    $Test->diag( $_ ) for @failures;
+    $TB->ok( $ok, $desc );
+    $TB->diag( $_ ) for @failures;
 
     return $ok;
 }
@@ -1072,10 +1129,11 @@ sub link_content_unlike {
     my $regex = shift;
     my $desc = shift;
 
-    my $usable_regex=$Test->maybe_regex( $regex );
-    unless(defined( $usable_regex )) {
-        my $ok = $Test->ok( 0, 'link_content_unlike' );
-        $Test->diag(qq{     "$regex" doesn't look much like a regex to me.});
+    my $usable_regex=$TB->maybe_regex( $regex );
+
+    if ( !defined( $usable_regex ) ) {
+        my $ok = $TB->ok( 0, 'link_content_unlike' );
+        $TB->diag(qq{     "$regex" doesn't look much like a regex to me.});
         return $ok;
     }
 
@@ -1084,8 +1142,8 @@ sub link_content_unlike {
     my @failures = $self->_check_links_content( \@urls, $regex, 'unlike' );
     my $ok = (@failures == 0);
 
-    $Test->ok( $ok, $desc );
-    $Test->diag( $_ ) for @failures;
+    $TB->ok( $ok, $desc );
+    $TB->diag( $_ ) for @failures;
 
     return $ok;
 }
@@ -1097,7 +1155,7 @@ sub _default_links_desc {
     return sprintf( '%d link%s %s', $url_count, $url_count == 1 ? '' : 's', $desc_suffix );
 }
 
-# This actually performs the status check of each url.
+# This actually performs the status check of each URL.
 sub _check_links_status {
     my $self = shift;
     my $urls = shift;
@@ -1116,7 +1174,7 @@ sub _check_links_status {
                 push( @failures, $url ) unless $mech->status() == $status;
             }
             else {
-                push( @failures, $url ) unless $mech->status() != $status;
+                push( @failures, $url ) if $mech->status() == $status;
             }
             $mech->back();
         }
@@ -1128,7 +1186,7 @@ sub _check_links_status {
     return @failures;
 }
 
-# This actually performs the content check of each url. 
+# This actually performs the content check of each URL.
 sub _check_links_content {
     my $self = shift;
     my $urls = shift;
@@ -1144,10 +1202,10 @@ sub _check_links_content {
         if ( $mech->follow_link( url => $url ) ) {
             my $content=$mech->content();
             if ( $test eq 'like' ) {
-                push( @failures, $url ) unless $content=~/$regex/;
+                push( @failures, $url ) unless $content =~ /$regex/;
             }
             else {
-                push( @failures, $url ) unless $content!~/$regex/;
+                push( @failures, $url ) if $content =~ /$regex/;
             }
             $mech->back();
         }
@@ -1165,8 +1223,9 @@ sub _format_links {
 
     my @urls;
     if (ref($links) eq 'ARRAY') {
-        if (defined($$links[0])) {
-            if (ref($$links[0]) eq 'WWW::Mechanize::Link') {
+        my $link = $links->[0];
+        if ( defined($link) ) {
+            if ( ref($link) eq 'WWW::Mechanize::Link' ) {
                 @urls = map { $_->url() } @{$links};
             }
             else {
@@ -1186,7 +1245,7 @@ Finds all free-text input fields (text, textarea, and password) in the
 current form and fills them to their maximum length in hopes of finding
 application code that can't handle it.  Fields with no maximum length
 and all textarea fields are set to 66000 bytes, which will often be
-enough to overflow the data's eventual recepticle.
+enough to overflow the data's eventual receptacle.
 
 There is no return value.
 
@@ -1317,11 +1376,367 @@ sub stuff_inputs {
     return;
 }
 
+
+
+=head2 $mech->lacks_uncapped_inputs( [$comment] )
+
+Executes a test to make sure that the current form content has no
+text input fields that lack the C<maxlength> attribute, and that each
+C<maxlength> value is a positive integer.  The test fails if the current
+form has such a field, and succeeds otherwise.
+
+Returns an array containing all text input fields in the current
+form that do not specify a maximum input length.  Fields for which
+the concept of input length is irrelevant, and controls that HTML
+does not allow to be capped (e.g. textarea) are ignored.
+
+The inputs in the returned array are descended from HTML::Form::Input.
+
+The return is true if the test succeeded, false otherwise.
+
+=cut
+
+sub lacks_uncapped_inputs {
+    my $self    = shift;
+    my $comment = shift;
+
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+
+    my @uncapped;
+
+    my @inputs = $self->grep_inputs( { type => qr/^(?:text|password)$/ } );
+    foreach my $field ( @inputs ) {
+        next if $field->readonly();
+        next if $field->disabled();
+
+        if ( not defined($field->{maxlength}) ) {
+            push( @uncapped, $field->name . ' has no maxlength attribute' );
+            next;
+        }
+
+        my $val = $field->{maxlength};
+        if ( ($val !~ /^\s*\d+\s*$/) || ($val+0 <= 0) ) {
+            push( @uncapped, $field->name . qq{ has an invalid maxlength attribute of "$val"} );
+        }
+    }
+
+    my $ok = $TB->cmp_ok( scalar @uncapped, '==', 0, $comment );
+    $TB->diag( $_ ) for @uncapped;
+
+    return $ok;
+}
+
+
+=head1 METHODS: MISCELLANEOUS
+
+=head2 $mech->autolint( [$status] )
+
+Without an argument, this method returns a true or false value indicating
+whether autolint is active.
+
+When passed an argument, autolint is turned on or off depending on whether
+the argument is true or false, and the previous autolint status is returned.
+As with the autolint option of C<< new >>, C<< $status >> can be an
+L<< HTML::Lint >> object.
+
+If autolint is currently using an L<< HTML::Lint >> object you provided,
+the return is that object, so you can change and exactly restore
+autolint status:
+
+    my $old_status = $mech->autolint( 0 );
+    ... operations that should not be linted ...
+    $mech->autolint( $old_status );
+
+=cut
+
+sub autolint {
+    my $self = shift;
+
+    my $ret = $self->{autolint};
+    if ( @_ ) {
+        $self->{autolint} = shift;
+    }
+
+    return $ret;
+}
+
+
+=head2 $mech->grep_inputs( \%properties )
+
+grep_inputs() returns an array of all the input controls in the
+current form whose properties match all of the regexes in $properties.
+The controls returned are all descended from HTML::Form::Input.
+
+If $properties is undef or empty then all inputs will be
+returned.
+
+If there is no current page, there is no form on the current
+page, or there are no submit controls in the current form
+then the return will be an empty array.
+
+    # get all text controls whose names begin with "customer"
+    my @customer_text_inputs =
+        $mech->grep_inputs( {
+            type => qr/^(text|textarea)$/,
+            name => qr/^customer/
+        }
+    );
+
+=cut
+
+sub grep_inputs {
+    my $self = shift;
+    my $properties = shift;
+
+    my @found;
+
+    my $form = $self->current_form();
+    if ( $form ) {
+        my @inputs = $form->inputs();
+        @found = _grep_hashes( \@inputs, $properties );
+    }
+
+    return @found;
+}
+
+
+=head2 $mech->grep_submits( \%properties )
+
+grep_submits() does the same thing as grep_inputs() except that
+it only returns controls that are submit controls, ignoring
+other types of input controls like text and checkboxes.
+
+=cut
+
+sub grep_submits {
+    my $self = shift;
+    my $properties = shift || {};
+
+    $properties->{type} = qr/^(?:submit|image)$/;  # submits only
+    my @found = $self->grep_inputs( $properties );
+
+    return @found;
+}
+
+# search an array of hashrefs, returning an array of the incoming
+# hashrefs that match *all* the pattern in $patterns.
+sub _grep_hashes {
+    my $hashes = shift;
+    my $patterns = shift || {};
+
+    my @found;
+
+    if ( ! %{$patterns} ) {
+        # nothing to match on, so return them all
+        @found = @{$hashes};
+    }
+    else {
+        foreach my $hash ( @{$hashes} ) {
+
+            # check every pattern for a match on the current hash
+            my $matches_everything = 1;
+            foreach my $pattern_key ( keys %{$patterns} ) {
+                $matches_everything = 0 unless exists $hash->{$pattern_key} && $hash->{$pattern_key} =~ $patterns->{$pattern_key};
+                last if !$matches_everything;
+            }
+
+            push @found, $hash if $matches_everything;
+        }
+    }
+
+    return @found;
+}
+
+
+=head2 $mech->scrape_text_by_attr( $attr, $attr_value [, $html ] )
+
+=head2 $mech->scrape_text_by_attr( $attr, $attr_regex [, $html ] )
+
+Returns an array of strings, each string the text surrounded by an
+element with attribute I<$attr> of value I<$value>.  You can also pass in
+a regular expression.  If nothing is found the return is an empty list.
+In scalar context the return is the first string found.
+
+If passed, I<$html> is scraped instead of the current page's content.
+
+=cut
+
+sub scrape_text_by_attr {
+    my $self = shift;
+    my $attr = shift;
+    my $value = shift;
+
+    my $html = $self->_get_optional_html( @_ );
+
+    my @results;
+
+    if ( defined $html ) {
+        my $parser = HTML::TokeParser->new(\$html);
+
+        while ( my $token = $parser->get_tag() ) {
+            if ( ref $token->[1] eq 'HASH' ) {
+                if ( exists $token->[1]->{$attr} ) {
+                    my $matched =
+                        (ref $value eq 'Regexp')
+                            ? $token->[1]->{$attr} =~ $value
+                            : $token->[1]->{$attr} eq $value;
+                    if ( $matched ) {
+                        my $tag = $token->[ 0 ];
+                        push @results, $parser->get_trimmed_text( "/$tag" );
+                        if ( !wantarray ) {
+                            last;
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    return $results[0] if !wantarray;
+    return @results;
+}
+
+
+=head2 scrape_text_by_id( $id [, $html ] )
+
+Finds all elements with the given ID attribute and pulls out the text that that element encloses.
+
+In list context, returns a list of all strings found. In scalar context, returns the first one found.
+
+If C<$html> is not provided then the current content is used.
+
+=cut
+
+sub scrape_text_by_id {
+    my $self = shift;
+    my $id   = shift;
+
+    my $html = $self->_get_optional_html( @_ );
+
+    my @results;
+
+    if ( defined $html ) {
+        my $found = index( $html, "id=\"$id\"" );
+        if ( $found >= 0 ) {
+            my $parser = HTML::TokeParser->new( \$html );
+
+            while ( my $token = $parser->get_tag() ) {
+                if ( ref $token->[1] eq 'HASH' ) {
+                    my $actual_id = $token->[1]->{id};
+                    $actual_id = '' unless defined $actual_id;
+                    if ( $actual_id eq $id ) {
+                        my $tag = $token->[ 0 ];
+                        push @results, $parser->get_trimmed_text( "/$tag" );
+                        if ( !wantarray ) {
+                            last;
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    return $results[0] if !wantarray;
+    return @results;
+}
+
+
+sub _get_optional_html {
+    my $self = shift;
+
+    my $html;
+    if ( @_ ) {
+        $html = shift;
+        assert_nonblank( $html, '$html passed in is a populated scalar' );
+    }
+    else {
+        if ( $self->is_html ) {
+            $html = $self->content();
+        }
+    }
+
+    return $html;
+}
+
+
+=head2 $mech->scraped_id_is( $id, $expected [, $msg] )
+
+Scrapes the current page for given ID and tests that it matches the expected value.
+
+=cut
+
+sub scraped_id_is {
+    my $self     = shift;
+    my $id       = shift;
+    my $expected = shift;
+    my $msg      = shift;
+
+    if ( not defined $msg ) {
+        my $what = defined( $expected ) ? $expected : '(undef)';
+
+        $msg = qq{scraped id "$id" is "$what"};
+    }
+
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+
+    my $got = $self->scrape_text_by_id($id);
+    is( $got, $expected, $msg );
+
+    return;
+}
+
+
+=head2 $mech->header_exists( $field [, $desc ] )
+
+Assures that a given response header exists. The actual value of the response header is not checked, only that the header exists.
+
+=cut
+
+sub header_exists {
+    my $self = shift;
+    my $field = shift;
+    my $desc = shift || qq{Response has $field header};
+
+    my $ok = defined($self->response->header($field));
+
+    $TB->ok( $ok, $desc );
+    if ( !$ok ) {
+        $TB->diag( HTTP::Headers::as_string($self->response) ) if $self->response;
+    }
+
+    return $ok;
+}
+
+=head2 $mech->header_matches( $field, $value [, $desc ] )
+
+Assures that a given response header exists and has the given value.  Value may be a string or a regular expression.
+
+=cut
+
+sub header_matches {
+    my $self = shift;
+    my $field = shift;
+    my $value = shift;
+    my $desc = shift || qq{Response has $field header with value '$value'};
+
+    my $actual_value = scalar $self->response->header($field);
+    my $ok = (ref($value) eq 'Regexp')
+       ? defined($actual_value) && ($actual_value =~ $value)
+       : defined($actual_value) && ($actual_value eq $value);
+
+    $TB->ok( $ok, $desc );
+    if ( !$ok ) {
+        $TB->diag( $self->response->header($field) ) if $self->response;
+    }
+    return $ok;
+}
+
+
 =head1 TODO
 
 Add HTML::Tidy capabilities.
 
-Add a broken image check.
+Other ideas for features are at https://github.com/petdance/test-www-mechanize
 
 =head1 AUTHOR
 
@@ -1330,9 +1745,7 @@ Andy Lester, C<< <andy at petdance.com> >>
 =head1 BUGS
 
 Please report any bugs or feature requests to
-<http://code.google.com/p/www-mechanize/issues/list>.  I will be
-notified, and then you'll automatically be notified of progress on
-your bug as I make changes.
+<https://github.com/petdance/test-www-mechanize>.
 
 =head1 SUPPORT
 
@@ -1344,13 +1757,9 @@ You can also look for information at:
 
 =over 4
 
-=item * Google Code bug tracker
+=item * Bug tracker
 
-L<http://code.google.com/p/www-mechanize/issues/list>
-
-Please B<do not use> the old queues for WWW::Mechanize and
-Test::WWW::Mechanize at
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Test-WWW-Mechanize>
+L<https://github.com/petdance/test-www-mechanize>
 
 =item * AnnoCPAN: Annotated CPAN documentation
 
@@ -1369,6 +1778,12 @@ L<http://search.cpan.org/dist/Test-WWW-Mechanize>
 =head1 ACKNOWLEDGEMENTS
 
 Thanks to
+Eric A. Zarko,
+moznion,
+Robert Stone,
+tynovsky,
+Jerry Gay,
+Jonathan "Duke" Leto,
 Philip G. Potter,
 Niko Tyni,
 Greg Sheard,
@@ -1383,20 +1798,10 @@ and Pete Krawczyk for patches.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2004-2011 Andy Lester.
+Copyright 2004-2016 Andy Lester.
 
-This program is free software; you can redistribute it and/or
-modify it under the terms of either:
-
-=over 4
-
-=item * the GNU General Public License as published by the Free
-Software Foundation; either version 1, or (at your option) any
-later version, or
-
-=item * the Artistic License version 2.0.
-
-=back
+This library is free software; you can redistribute it and/or modify it
+under the terms of the Artistic License version 2.0.
 
 =cut
 
